#include <msd/bucket.h>
#include <unistd.h>
#include <list>

using std::list;

static list<Bucket *> bucketList;

Bucket *Bucket::ref()
{
	mRefCount++;
}

void Bucket::free()
{
	assert(mRefCount > 0);
	mRefCount--;
}

Bucket *Bucket::allocate()
{
	list<Bucket *>::iterator i;
	int bestCapacity = 0;
	Bucket *bestBucket = 0;

	for(i = bucketList.begin(); i != bucketList.end(); i++)
	{
		Bucket *b = *i;

		if(!b->exclusive() && b->capacity() > bestCapacity)
		{
			bestBucket = b;
			bestCapacity = b->capacity();
		}

		/* already a perfect match? */
		if(b->used() == 0)
			return b->ref();
	}

	if(!bestBucket || bestBucket->capacity() < 512)
	{
		bestBucket = new Bucket();
		bucketList.push_back(bestBucket);
	}
	return bestBucket->ref();
}

Bucket *Bucket::allocateEmpty()
{
	Bucket *b = allocate();

	if(b->used() != 0)
	{
		b->free();
		b = new Bucket();
		bucketList.push_back(b);
	}
	return b->ref();
}

Bucket::Bucket()
	: mUsed(0), mRefCount(0)
{
}

int Bucket::readSocketData(int fd)
{
	assert(mExclusive);

	int bytes = read(fd,&mData[mUsed],capacity());

	if(bytes > 0) mUsed += bytes;
	return bytes;
}

int Bucket::readBucketData(Bucket *bucket, int start, int len)
{
	assert(mExclusive);
	assert(bucket->used() >= start + len);
	assert(start >= 0);
	assert(len >= 0);

	int bytes = min(capacity(), len);

	memcpy(&mData[mUsed], bucket->data() + start, bytes);
	return bytes;
}
