#ifndef MSD_BUCKET_H
#define MSD_BUCKET_H

#include <msd/msd.h>
#include <assert.h>

class Bucket {
private:
	unsigned char mData[MSD_MAX_MSG_SIZE];
	int mUsed;
	int mRefCount;
	bool mExclusive;

protected:
	Bucket();
	~Bucket();

public:
	unsigned const char *data() const { return mData; }
	int capacity() const { return MSD_MAX_MSG_SIZE - mUsed; }
	int used() const { return mUsed; }

	bool exclusive() const { return mExclusive; }
	void setExclusive(bool newExclusive) {
		assert(newExclusive != mExclusive);
		mExclusive = newExclusive;
	}

	static Bucket *allocate();
	static Bucket *allocateEmpty();

	Bucket *ref();
	void free();
	int readSocketData(int fd);
	int readBucketData(Bucket *b, int start, int len);

	unsigned char *write(int len) {
		unsigned char *result = &mData[mUsed];
		assert(mExclusive);
		assert(len >= 0);
		assert(mUsed + len <= MSD_MAX_MSG_SIZE);
		mUsed += len;
		return result;
	}
};

#endif /* MSD_BUCKET_H */
