#include <msd/client.h>
#include <msd/dispatcher.h>
#include <msd/bucket.h>
#include <msd/demarshaller.h>
#include <msd/marshaller.h>
#include <msd/debug.h>
#include <stdio.h>

using namespace MSD;

Client::Client(int fd)
{
	mFD = fd;
	mBucket = 0;
	Dispatcher::the()->ioManager()->watchFD(fd, IOType::read, this);
}

void Client::notifyIO(int fd, int types)
{
	assert(fd == mFD);
	assert(types & IOType::read);

	/* if necessary, get new bucket */
	if(!mBucket)
	{
		mBucket = Bucket::allocate();
		mBucket->setExclusive(true);

		mMessageStart = mBucket->used();
	}

	/* read some data */
	int rc = mBucket->readSocketData(fd);
	/*
	int remaining = mBucket->used() - mMessageStart;
	const unsigned char *data = mBucket->data() + mMessageStart;

	printf("got something (");
	fflush(stdout);
	write(1,data,remaining);
	printf(").\n");
	*/

	/* process complete messages */

	Demarshaller demarshaller(mBucket, mMessageStart);
	while(demarshaller.remaining() >= 16)
	{
		printf("<before> remaining = %d\n",demarshaller.remaining());
		int magic = demarshaller.readInt();
		int type = demarshaller.readInt();
		int length = demarshaller.readInt();
		int msgid = demarshaller.readInt();

		printf("<after header> remaining = %d\n",demarshaller.remaining());
		if((demarshaller.remaining() + 16) < length)
		{
			printf("[partial message] remaining = %d, length = %d\n",
									  demarshaller.remaining(), length);
			break;
		}

		printf("[message] magic = %d, type = %d, len = %d, msgid = %d\n",
				magic, type, length, msgid);

		if(magic != MSD_MSG_MAGIC)
		{
			printf("MSD: communication error with client on socket %d.\n", mFD);
			mBucket->setExclusive(false);
			mBucket->free();
			mBucket = 0;
			return;
		}

		switch(type)
		{
			case MSD_MSG_STREAM_OPEN:
				{
					int stream = demarshaller.readInt();
					int rate = demarshaller.readInt();
					int channels = demarshaller.readInt();
					int format = demarshaller.readInt();

					streamOpen(stream, rate, channels, format);
				}
				break;
		}
		printf("<after> remaining = %d\n",demarshaller.remaining());

		mMessageStart += length;
		if(demarshaller.pos() != mMessageStart || demarshaller.error())
		{
			arts_warning("demarshalling error");
		}
	}
	
	/* at message boundary? */
	if(mMessageStart == mBucket->used())
	{
		mBucket->setExclusive(false);
		mBucket->free();
		mBucket = 0;
	}
	else
	{
		/* move partial message data into a new bucket */
		if(mMessageStart != 0)
		{
			Bucket *partial = mBucket;
			mBucket = Bucket::allocateEmpty();
			mBucket->setExclusive(true);
			mBucket->readBucketData(partial,
					                mMessageStart,
									mBucket->used()-mMessageStart);
			mMessageStart = 0;
			partial->setExclusive(false);
			partial->free();
		}
	}
}

void Client::streamOpen(int stream, int rate, int channels, int format)
{
	printf("got streamOpen stream=%d, rate=%d, channels=%d, format=%d\n",
			stream,rate,channels,format);

	Bucket *response = Bucket::allocateEmpty();
	response->setExclusive(true);

	Marshaller marshaller(response);
	marshaller.writeInt(MSD_MSG_MAGIC);
	marshaller.writeInt(16 /* FIXME */);
	marshaller.writeInt(MSD_MSG_RESULT);
	marshaller.writeInt(0);

	int result = write(mFD,response->data(),response->used());
	if(result != response->used())
	{
		printf("baaad\n");
	}
}
