#include <msd/msd.h>
#include <msd/bucket.h>

namespace MSD {

class Demarshaller {
private:
	const Bucket *mBucket;
	int mPos;
	bool mError;

public:
	Demarshaller(const Bucket *bucket, int start)
		: mBucket(bucket), mPos(start), mError(false)
	{
	}

	int readInt() {
		if(mPos + 4 > mBucket->used())
		{
			mError = true;
			return 0;
		}
		const unsigned char *data = mBucket->data() + mPos;
		mPos += 4;
		return (data[0] << 24) + (data[1] << 16) + (data[2] << 8) + data[3];
	}

	int64 readInt64() {
		if(mPos + 8 > mBucket->used())
		{
			mError = true;
			return 0;
		}
		const unsigned char *data = mBucket->data() + mPos;
		mPos += 8;
		return ((int64)data[0] << 56) + ((int64)data[1] << 48) +
			   ((int64)data[2] << 40) + ((int64)data[3] << 32) +
			   ((int64)data[4] << 24) + ((int64)data[5] << 16) +
			   ((int64)data[6] <<  8) + ((int64)data[7]);
	}

	const unsigned char *readByteArray(int& length)
	{
		length = readInt();
		if(mError || (mPos + length) > mBucket->used())
		{
			length = 0;
			mError = true;
			return 0;
		}
		const unsigned char *data = mBucket->data() + mPos;
		mPos += length;
		return data;
	}

	int remaining() const {
		return max(mBucket->used() - mPos, 0);
	}
	int pos() const {
		return mPos;
	}
	bool error() const {
		return mError;
	}
};

}
